package net.anotheria.anouitest.page;

import org.jbehave.web.selenium.WebDriverProvider;
import org.reflections.Reflections;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Map;

/**
 * Page factory contains all pages.
 *
 * @author ivanbatura
 */
public class PageFactory {
    /**
     * All Pages.
     */
    private Map<String, Page> pages;

    /**
     * Get all pages from factory.
     *
     * @return pages.
     */
    public Map<String, Page> getPages() {
        return pages;
    }

    /**
     * Constructor.
     *
     * @param driverProvider {@link WebDriverProvider}
     */
    public PageFactory(WebDriverProvider driverProvider) {
        this.pages = new HashMap<String, Page>();
        for (Class<?> type : new Reflections("").getSubTypesOf(Page.class)) {
            if (!type.isInterface() && !Modifier.isAbstract(type.getModifiers())) {
                try {
                    Constructor ctor = type.getDeclaredConstructor(WebDriverProvider.class);
                    Page page = (Page) ctor.newInstance(driverProvider);
                    pages.put(page.getPageName(), page);

                } catch (NoSuchMethodException e) {
                    throw new RuntimeException("Can not add page" + type.getName() + " to PageFactory", e);
                } catch (InvocationTargetException e) {
                    throw new RuntimeException("Can not add page" + type.getName() + " to PageFactory", e);
                } catch (InstantiationException e) {
                    throw new RuntimeException("Can not add page" + type.getName() + " to PageFactory", e);
                } catch (IllegalAccessException e) {
                    throw new RuntimeException("Can not add page" + type.getName() + " to PageFactory", e);
                }
            }
        }
    }

    /**
     * Reset pages.
     */
    public void reset() {
        pages = null;
    }
}
