package net.anotheria.anouitest.settings;

import net.anotheria.anouitest.webserver.EmbeddedServerType;
import org.configureme.ConfigurationManager;
import org.configureme.annotations.ConfigureMe;

/**
 * Config for embedded server. Needed only if you use embedded server.
 *
 * @author ivanbatura
 */
@ConfigureMe(name = "embedded-server", allfields = true)
public class EmbeddedServerConfig {

	/**
	 * Use embedded server or not.
	 */
	private boolean useEmbeddedServer;
	/**
	 * Location of the war in the file system in target folder.
	 */
	private String pathToWar;
	/**
	 * Web server port
	 */
	private int webServerPort;

	/**
	 * Catalina home folder.
	 */
	private String catalinaHome;

	/**
	 * Embedded server type.
	 */
	private EmbeddedServerType embeddedServerType;

	/**
	 * If embedded server should be lunched in other thread.
	 */
	private boolean lunchServerInOtherThread;

	/**
	 * Web application name in order to be able to deploy it to the embedded server.
	 */
	private String webAppName;

	public boolean isUseEmbeddedServer() {
		return useEmbeddedServer;
	}

	public void setUseEmbeddedServer(boolean useEmbeddedServer) {
		this.useEmbeddedServer = useEmbeddedServer;
	}

	public String getPathToWar() {
		return pathToWar;
	}

	public void setPathToWar(String pathToWar) {
		this.pathToWar = pathToWar;
	}

	public int getWebServerPort() {
		return webServerPort;
	}

	public void setWebServerPort(int webServerPort) {
		this.webServerPort = webServerPort;
	}

	public String getCatalinaHome() {
		return catalinaHome;
	}

	public void setCatalinaHome(String catalinaHome) {
		this.catalinaHome = catalinaHome;
	}

	public EmbeddedServerType getEmbeddedServerType() {
		return embeddedServerType;
	}

	public void setEmbeddedServerType(EmbeddedServerType embeddedServerType) {
		this.embeddedServerType = embeddedServerType;
	}

	public boolean isLunchServerInOtherThread() {
		return lunchServerInOtherThread;
	}

	public void setLunchServerInOtherThread(boolean lunchServerInOtherThread) {
		this.lunchServerInOtherThread = lunchServerInOtherThread;
	}

	public static EmbeddedServerConfig getInstance() {
		return Holder.INSTANCE;
	}

	public String getWebAppName() {
		return webAppName;
	}

	public void setWebAppName(String webAppName) {
		this.webAppName = webAppName;
	}

	private static class Holder {
		/**
		 * Instance of pace config.
		 */
		private static final EmbeddedServerConfig INSTANCE;

		static {
			INSTANCE = new EmbeddedServerConfig();
			try {
				ConfigurationManager.INSTANCE.configure(INSTANCE);
			} catch (final Exception e) {
				INSTANCE.setCatalinaHome(System.getProperty("java.io.tmpdir") + "/embedded-server");
				INSTANCE.setUseEmbeddedServer(false);
				INSTANCE.setWebServerPort(8080);
				INSTANCE.setEmbeddedServerType(EmbeddedServerType.TOMCAT);
				INSTANCE.setLunchServerInOtherThread(true);
				INSTANCE.setWebAppName("embedded");
			}
		}
	}
}
