package net.anotheria.anouitest.settings;

import net.anotheria.anouitest.core.RunnableBrowser;
import org.configureme.ConfigurationManager;
import org.configureme.annotations.ConfigureMe;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * General Configuration for ano-uitest.
 *
 * @author ivanbatura
 */
@ConfigureMe(name = "ui-test-config", allfields = true)
public class GeneralConfig {
    /**
     * Default Ajax timeout.
     */
    public static final int AJAX_ELEMENT_TIMEOUT = 20;
    /**
     * Base application url.
     */
    private String baseUrl;

    /**
     * Web Drivers for selenium.
     */
    private RunnableBrowser[] runnableDrivers;

    /**
     * Excluded scenarios from test by meta data
     */
    private String[] excludedScenariosByMeta;

    /**
     * Included scenarios from test by meta data
     */
    private String[] includedScenariosByMeta;

    /**
     * Date format default is  dd/MM/yyyy)
     */
    private String defaultDateFormat;

    /**
     * The code below waits for this value or till the element is available, whichever is the earliest.
     */
    private int ajaxElementTimeout;

    public static GeneralConfig getInstance() {
        return Holder.INSTANCE;
    }

    public String getBaseUrl() {
        return baseUrl;
    }

    public void setBaseUrl(String baseUrl) {
        this.baseUrl = baseUrl;
    }

    public RunnableBrowser[] getRunnableDrivers() {
        return runnableDrivers;
    }

    public void setRunnableDrivers(RunnableBrowser[] runnableDrivers) {
        this.runnableDrivers = runnableDrivers;
    }

    public String[] getExcludedScenariosByMeta() {
        return excludedScenariosByMeta == null ? new String[]{} : excludedScenariosByMeta;
    }

    public void setExcludedScenariosByMeta(String[] excludedScenariosByMeta) {
        this.excludedScenariosByMeta = excludedScenariosByMeta;
    }

    public String[] getIncludedScenariosByMeta() {
        return includedScenariosByMeta == null ? new String[]{} : includedScenariosByMeta;
    }

    public void setIncludedScenariosByMeta(String[] includedScenariosByMeta) {
        this.includedScenariosByMeta = includedScenariosByMeta;
    }

    public String getDefaultDateFormat() {
        return defaultDateFormat;
    }

    public void setDefaultDateFormat(String defaultDateFormat) {
        this.defaultDateFormat = defaultDateFormat;
    }

    public int getAjaxElementTimeout() {
        return ajaxElementTimeout;
    }

    public void setAjaxElementTimeout(int ajaxElementTimeout) {
        this.ajaxElementTimeout = ajaxElementTimeout;
    }

    /**
     * Inner config instance holder.
     */
    private static class Holder {
        /**
         * Instance of pace config.
         */
        private static final GeneralConfig INSTANCE;

        static {
            INSTANCE = new GeneralConfig();
            try {
                ConfigurationManager.INSTANCE.configure(INSTANCE);
            } catch (final Exception e) {
                INSTANCE.setBaseUrl("http://localhost:8080");
                List<RunnableBrowser> driverList = new ArrayList<RunnableBrowser>(Arrays.asList(RunnableBrowser.HTMLUNIT));
                INSTANCE.setRunnableDrivers(driverList.toArray(new RunnableBrowser[driverList.size()]));
                INSTANCE.setDefaultDateFormat("dd/MM/yyyy");
                INSTANCE.setAjaxElementTimeout(AJAX_ELEMENT_TIMEOUT);
            }
        }
    }
}
