package net.anotheria.anouitest.webserver;

import net.anotheria.anouitest.settings.EmbeddedServerConfig;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.core.StandardHost;
import org.apache.catalina.startup.Tomcat;
import org.apache.commons.io.FileUtils;

import java.io.File;
import java.io.IOException;

/**
 * Embedded web server - Tomcat.
 *
 * @author ivanbatura
 */
public class EmbeddedTomcat implements EmbeddedServer {
    /**
     * Embedded tomcat.
     */
    private final Tomcat tomcat;
    /**
     * Folder name for web applications.
     */
    private static final String WEBAPP = "webapps";
    /**
     * Catalina home.
     */
    private final File catalinaHome;

    public EmbeddedTomcat() {
        EmbeddedServerConfig config = EmbeddedServerConfig.getInstance();
        this.catalinaHome = new File(config.getCatalinaHome());

        tomcat = new Tomcat();
        tomcat.setPort(config.getWebServerPort());
        tomcat.setBaseDir(catalinaHome.getAbsolutePath());
        tomcat.getHost().setCreateDirs(true);
        tomcat.getHost().setAppBase(WEBAPP);
        tomcat.getHost().setDeployOnStartup(true);
        tomcat.getHost().setAutoDeploy(true);
        ((StandardHost) tomcat.getHost()).setWorkDir(catalinaHome.getAbsolutePath() + "/" + WEBAPP);
        ((StandardHost) tomcat.getHost()).setUnpackWARs(true);
    }

    @Override
    public void start() {
        try {
            tomcat.start();
        } catch (LifecycleException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void stop() {
        try {
            tomcat.stop();
            tomcat.destroy();
            FileUtils.deleteDirectory(catalinaHome);
        } catch (LifecycleException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            //ignore
        }
    }

    @Override
    public void deployWar(String appName, File file) {
        try {
            FileUtils.copyFile(file, new File(catalinaHome.getAbsolutePath() + "/" + WEBAPP + "/" + file.getName()));
        } catch (IOException e) {
            throw new IllegalArgumentException("File " + file.getAbsolutePath() + " can not copy to web application name=" + appName, e);
        }
        deploy(appName);
    }

    @Override
    public String getApplicationUrl(String appName) {
        return String.format("http://%s:%d/%s", tomcat.getHost().getName(),
                tomcat.getConnector().getLocalPort(), appName);
    }

    /**
     * Deploy application to server.
     *
     * @param appName application name on the server
     */
    private void deploy(String appName) {
        tomcat.addWebapp(null, "/" + appName, catalinaHome.getAbsolutePath() + "/" + WEBAPP + "/" + appName);
    }
}